/**
 * 
 * Last changed: 2010-4-22 By: ws added @Override's URL: $HeadURL$ $Revision$
 *
 */
package de.desy.mst.libs.framework.mstapp.appinf.jaws;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URISyntaxException;
import java.net.URL;

import de.desy.mst.libs.framework.mstapp.appinf.AApp;
import de.desy.mst.libs.framework.mstapp.appinf.AApplicationInterfaceFactory;
import de.desy.mst.libs.framework.mstapp.appinf.IApp;

/**
 * @author labudda
 *
 */
public class JawsFactory extends AApplicationInterfaceFactory {
  private final static java.util.logging.Logger log = java.util.logging.Logger.getLogger(JawsFactory.class.getName());  
  private final java.util.concurrent.atomic.AtomicReference<IApp> appRef;

  public JawsFactory(){
    this.appRef = new java.util.concurrent.atomic.AtomicReference<IApp>();
  }
  /*
   * (non-Javadoc)
   * 
   * @see de.desy.mst.libs.IApplikationInterfaceFactory#getApplicationInformation()
   */
  @Override
  public IApp getApplicationInformation() throws IllegalStateException {
    final IApp app = this.getApp();
    if (app.isInformationValid()) {
      return app;
    } else {
      throw new IllegalStateException("This is not detected as a valid jaws application");
    }
  }

  /**
   * @return a App info, might be invalid. Never Null
   */
  private IApp getApp() {
    IApp app = this.appRef.get();
    if (app == null) {
      synchronized (this.appRef) {
        app = this.appRef.get();
        if (app == null) {
          app = JawsFactory.loadAppInfo();
          this.appRef.set(app);
          return app;
        } else {
          return app;
        }
      }
    } else {
      return app;
    }
  }

  /**
   * Versucht die jnlp Datei dieser Applikation zu lesen Die jnlp Datei ist ggf. in der system
   * property jaws.href definiert Wenn die Property nicht definiert ist oder die Datei nicht gelesen
   * werden kann ist das Ergebnis null
   * 
   * @return app Info Object or Null App Info
   */
  private static IApp loadAppInfo() {
    final String fileLocation = System.getProperty("jaws.href", null);
    if (fileLocation == null) {
      /* Logging als Info und nicht als Warnung, weil die Ermittlung
       * des Typs der Applikation / Factory auf der Abfrage dieser Property
       * basiert
      */ 
      JawsFactory.log.info("jaws.href not defined");
      return AApp.Null;
    } else {
      try {
        final java.net.URL url = new java.net.URL(fileLocation);

        try (final java.io.InputStream is = JawsFactory.openStream(url);) {
          final javax.xml.parsers.DocumentBuilderFactory factory =
              javax.xml.parsers.DocumentBuilderFactory.newInstance();

          factory.setNamespaceAware(true);
          javax.xml.parsers.DocumentBuilder db = factory.newDocumentBuilder();
          final org.w3c.dom.Document doc = db.parse(is);
          JawsFactory.log.fine(fileLocation + " parsed as valid xml file");
          return new JawsApp(doc);
        } catch (java.io.IOException | URISyntaxException p) {
          JawsFactory.log.log(java.util.logging.Level.WARNING, "Unable to load " + fileLocation, p);
          return AApp.Null;
        }

      } catch (MalformedURLException | org.xml.sax.SAXException | IllegalArgumentException
          | javax.xml.parsers.ParserConfigurationException e) {
        JawsFactory.log.log(java.util.logging.Level.WARNING, "Unable to get url " + fileLocation, e);
        return AApp.Null;
      }
    }
  }

  /**
   * @param url to open
   * @return an open stream to be closed outside
   * @throws IOException
   * @throws URISyntaxException
   */
  private static InputStream openStream(URL url) throws IOException, URISyntaxException {
    final String protokoll = url.getProtocol().toLowerCase();
    if (protokoll.equals("http") || protokoll.equals("https")) {
      return url.openStream();
    } else if (protokoll.equals("file")) {
      java.io.File file = new java.io.File(url.toURI());
      return new java.io.FileInputStream(file);
    } else {
      throw new ProtocolException(protokoll + " is not supported");
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.desy.mst.libs.framework.mstapp.appinf.AApplicationInterfaceFactory#
   * areKindSpecificRequirementsFulfilled()
   */
  @Override
  protected boolean areKindSpecificRequirementsFulfilled() {
    return this.getApp().isInformationValid();
  }
}
